//-------------------------------------------------------------------------------------------------
/// \file	SimpleNode.cpp
/// \date	6-11-04
/// \author	Rob Bateman
/// \brief	This file impliments a simple node within Maya
///
//-------------------------------------------------------------------------------------------------
#include <math.h>
#include "SimpleNode.h"

const MTypeId SimpleNode::m_TypeId( 0x70032 );
const MString SimpleNode::m_TypeName( "simpleNode" );


/// an connection made to the input time value
MObject SimpleNode::m_aTime;

/// A single output floating point value
MObject SimpleNode::m_aOutput;

/// a compound attribute for our graph
MObject SimpleNode::m_aGraph;

//-------------------------------------------------------------------------------------------------
//
void* SimpleNode::creator()
{
	// return a new instance of the node for maya to use
    return new SimpleNode();
}

//-------------------------------------------------------------------------------------------------
//
MStatus SimpleNode::initialize()
{
	// the input attibute is a time attribute that will be connected to the global time node attribute,
	// time1.outTime
	//
	//	Any attribute types that are measured in changeable units, ie distance(cm, mm, miles, feet),
	//	angles( degrees, radians ), time( secs, mins, frames ) etc should be created using the
	//	MFnUnitAttribute function set.
	//
	MFnUnitAttribute uAttr;

    // create the input time attribute
	m_aTime =  uAttr.create( "time", "t", MFnUnitAttribute::kTime );

	// since the input attribute will be connected to the time value, there is little point
	// in storing this attribute within the maya binary file. There is also no point in
	// making the attribute keyable.
	uAttr.setStorable(false);
	uAttr.setKeyable(false);


	//	The MFnNumericAttribute function set allows us to add numeric types such as
	//
	//	bool, int, float, double, point, colour etc.
	//
    MFnNumericAttribute nAttr;

    // create the output attribute
    m_aOutput = nAttr.create( "output", "o",
						   MFnNumericData::kFloat,
						   0.0f);

	// make this attribute a read only output value
	nAttr.setReadable(true);
    nAttr.setKeyable(false);
	nAttr.setWritable(false);


	// add the attributes to the node
	addAttribute(m_aTime);
	addAttribute(m_aOutput);

	// set the attribute dependency. the output attribute will now be evaluated whenever
	// the time input changes.
	//
	attributeAffects(m_aTime,  m_aOutput);


	//---------------------------------------------------------------------- Graph Attr.
	////////////////////////////////////////////////////////////////////////
	//
	// repeat for each graph attribute required...
	//
    MFnCompoundAttribute cAttr;
    MFnEnumAttribute eAttr;

	// create a compound attribute
	m_aGraph = cAttr.create( "graph", "gr" );

	// make it an array of the following structures...
	cAttr.setArray(true);

		// so that each graph point has the following attributes:

		// create a position attribute, NOTE NAMING CONVENTION!   (x axis, 0 to 1)
		MObject oPosition = nAttr.create( "graph_Position","grp", MFnNumericData::kFloat, 0.0f);
		nAttr.setReadable(true);
		nAttr.setKeyable(true);
		nAttr.setWritable(true);

		// create a value attribute, NOTE NAMING CONVENTION!      (y axis, 0 to 1)
		MObject oFloatValue = nAttr.create( "graph_FloatValue","grv", MFnNumericData::kFloat, 0.0f);
		nAttr.setReadable(true);
		nAttr.setKeyable(true);
		nAttr.setWritable(true);


		// create an enum attribute for the interpolation type, NOTE NAMING CONVENTION!
		MObject oEnumValue = eAttr.create( "graph_Interp","gri" );

		// add enum values
		eAttr.addField("None",0);
		eAttr.addField("Linear",1);
		eAttr.addField("Smooth",2);
		eAttr.addField("Spline",3);
		eAttr.setReadable(true);
		eAttr.setKeyable(true);
		eAttr.setWritable(true);

	// add children to compound attribute
	cAttr.addChild( oPosition );
	cAttr.addChild( oFloatValue );
	cAttr.addChild( oEnumValue );

	// add attribute to node
	addAttribute(m_aGraph);

	////////////////////////////////////////////////////////////////////////
	//----------------------------------------------------------------------

	// done
    return MS::kSuccess;
}

//-------------------------------------------------------------------------------------------------
//
//
MStatus SimpleNode::compute (const MPlug& plug, MDataBlock& data)
{
	// Check that the requested recompute is one of the output values
	//
	if( plug != m_aOutput )
		return MS::kUnknownParameter;

	// get the input time handle
	MDataHandle timeData = data.inputValue(m_aTime);

	// get the output handle
	MDataHandle outputData = data.outputValue(m_aOutput);

	// get the input value as a time value
	MTime Time = timeData.asTime();

	// set the output to the sine of the time (in seconds)
	outputData.set( (float)sin( Time.as(MTime::kSeconds) ) );

	// clean the output plug, ie unset it from dirty so that maya does not re-evaluate it
	data.setClean( plug );

	return MS::kSuccess;
}
